################################################################################
# Licensed Materials - Property of IBM Copyright IBM Corporation 2012. All Rights Reserved.
# U.S. Government Users Restricted Rights - Use, duplication or disclosure restricted by GSA ADP
# Schedule Contract with IBM Corp.
################################################################################

require "devops_build/providers/build_system_provider"

module DevopsBuild
  module Providers

    # MockRtcBuildSystemProvider - an implementation of the BuildSystemProvider
    # interface that pretends to interact with the Rational Team Concert build system.
    #
    # For documentation on what each method does, please see the comments in 
    # com.ibm.devops.cloud/lib/build/providers/build_system_provider.rb
    #
    class MockRtcBuildSystemProvider < BuildSystemProvider
  
      # See RtcBuildSystemProvider#initialize
      def initialize(repository_url, user_id, password_hash={})
        @activity_id = 0
        @rtc_args = {
          :repositoryAddress => repository_url,
          :userId => user_id
        }.merge(password_hash)
        
      end
      
      # See RtcBuildSystemProvider#resolve_build
      def resolve_build(uri, uuid, id, type=nil, environment=nil, build_requestor_id=nil)
        return Struct.new(:uri, :uuid, :id, :type, :environment, :build_requestor_id).new(uri, uuid, id, type, environment, build_requestor_id)
      end
      
      # See BuildSystemProvider#start_activity
      def start_activity(build, label, parent_activity=nil)
        return start_activity_internal(build, label, parent_activity, false)
      end

      # See BuildSystemProvider#complete_activity
      def complete_activity(activity)
        return nil
      end
  
      # See BuildSystemProvider#publish_activity
      def publish_activity(build, label, parent_activity=nil)
        start_activity_internal(build, label, parent_activity, true)
      end
  
      # See BuildSystemProvider#publish_artifact_link
      def publish_artifact_link(build, label, url, local_file_path=nil)
        return nil
      end
      
      # See BuildSystemProvider#publish_link
      def publish_link(build, url, label, componentName=nil)
        return nil
      end
      
      # See RtcBuildSystemProvider#publish_junit_results      
      def publish_junit_results(build, junit_result_file)
        return nil
      end
      
      # See RtcBuildSystemProvider#publish_cloud_resources      
          def publish_cloud_resources(build, instance_id, cloud_user_name, cloud_password, cloud_url)
            return nil
          end
      
      # See RtcBuildSystemProvider#publish_cloud_logs
          def publish_cloud_logs(build, instance_id, cloud_user_name, cloud_password, cloud_url)
            return nil
          end
          
      # See RtcBuildSystemProvider#publish_log_file      
      def publish_log_file(build, label, log_file, status = 'OK', content_type = 'text/plain')
        return nil
      end
  
      private

      #################
      # Private Methods
      
      def start_activity_internal(build, label, parent_activity, auto_complete)
        @activity_id = @activity_id + 1
        unless auto_complete
          return Struct.new(:id, :build).new(@activity_id, build)
        end
      end
  
    end # end RtcProvider
  end # end module Providers
end # end module DevopsBuild
