module Weaver
  # \Weaver::Generators provides tools to generate \Weaver files for various roles
  #
  # @see EclipseAppGenerator
  # @see EnvironmentGenerator
  # @see InfFacadeGenerator
  module Generators
    # NodeGenerator generates a fully 'mixed' {Weaver::Core::Node} as it would be after
    # it has been linked with a {Weaver::Core::Provider}.
    class NodeGenerator

      # Cleans a node so the information recreated by mixing is no longer present
      #
      # * *Args*    :
      #   - +node+ -> a String; the name of the pattern Infrastructure
      def self.account_for_mixing(node)
        # To calculate which attributes will be mixed, mix a dummy empty node too
        dummy_node = node.clone
        dummy_node.amd.clear
        node.provisioner.mixin_provider_attributes(dummy_node)
        mixed_attributes = dummy_node.amd.keys
        
        # Ask for provider-specific attributes, but when they arrive remove
        # their validators and provider-name because those will be re-mixed anyway upon parse.
        node.provisioner.mixin_provider_attributes(node)
        node.amd.each do |id, attr| 
          if mixed_attributes.member?(id)
            attr.implicit = false
            attr.name = nil
            attr.instance_eval { @validators.clear }
          end
        end
      end

    end

  end

end
