require 'rest_client'
require 'weaver/service/rest_client_base'
require 'weaver/service/weaver_service_error'

module Weaver
  module Service
    # Implements methods that make REST calls on an endpoint using RestClient, initialized with a host and port
    class RestClientBase

      APPLICATION_JSON = 'application/json'

      def initialize(host, port)
        port = nil if ((port.kind_of? String) && port.empty?)
        @uri = "http://#{host}"
        @uri << ":#{port}" if !port.nil?
      end

      def get(uri)
        begin
          RestClient.get endpoint(uri), :content_type => APPLICATION_JSON
        rescue Errno::ECONNREFUSED => e
          raise_error(e.message, e)
        rescue => e
          raise_error(e.message, e)
        end
      end

      def post(uri, payload)
        begin
          RestClient.post endpoint(uri), payload, :content_type => APPLICATION_JSON
        rescue RestClient::Exception => e
          raise_error(e.message, e)
        rescue => e
          raise_error(e.message, e)
        end
      end

      def put(uri, payload = nil)
        begin
          RestClient.put endpoint(uri), payload, :content_type => APPLICATION_JSON
        rescue Errno::ECONNREFUSED => e
          raise_error(e.message, e)
        rescue => e
          raise_error(e.message, e)
        end
      end

      def delete(uri)
        begin
          RestClient.delete endpoint(uri), :content_type => APPLICATION_JSON
        rescue Errno::ECONNREFUSED => e
          raise_error(e.message, e)
        rescue => e
          raise_error(e.message, e)
        end
      end

      private

      def endpoint(uri)
        if uri.index('/') != 0 # handling missing '/' at the front
          uri = "/#{uri}"
        end
        "#{@uri}#{uri}"
      end

      def raise_error(msg, e)
        puts("@@@TR in raise_error in WeaverService, message is #{msg}, exception class is #{e.class}, exception is #{e.inspect}")
        raise Weaver::Service::WeaverServiceError.new(msg, nil, e)
      end

    end

  end
end