module Weaver
  module Utils
    # This class makes the relationship Group.member and Base.member_of stay in sync
    # 
    # Author:: Ed Snible
    class ReversingHash
      def initialize(entity, reverse_relationship_type)
        @entity = entity
        @reverse_relationship_type = reverse_relationship_type
        @h = {}
      end
      
      def []=(key, newval)
        unless @updating    # Prevent unbounded indirect recursion
          @updating = true
          
          oldval = @h[key]
          @h[key] = newval
           
          # Remove the old if there was one and we are changing it
          unless oldval.nil? || oldval.equal?(newval)
            reverse_relationship = oldval.send @reverse_relationship_type
            reverse_relationship.delete(@entity.id) unless reverse_relationship.nil?
          end
          
          # Add the new if there is one and we are changing it
          
          unless newval.nil? || newval.equal?(oldval)
            reverse_relationship = newval.send @reverse_relationship_type unless newval.nil?
            reverse_relationship[@entity.id] = @entity unless reverse_relationship.nil?
          end
          
          @updating = false
        end
      end
      
      def delete(key)
        unless @updating    # Prevent unbounded indirect recursion
          @updating = true
          
          member = @h.delete(key)
          
          reverse_relationship = member.send @reverse_relationship_type unless member.nil?
          reverse_relationship.delete(@entity.id) unless reverse_relationship.nil?
          @updating = false
        end
      end
      
      def size()
        return @h.size()
      end # def size
      
      alias :length size
      
      def empty?()
        return @h.empty?()
      end # def empty?
      
      def method_missing(sym, *arguments, &block)
        @h.send sym, *arguments, &block
      end
    end
  end
end
