require 'json'
require 'weaver/continuous_validation'
require 'weaver/continuous_validation/rest_client_base'
require 'weaver/continuous_validation/weaver_service_error'

module Weaver
  module ContinuousValidation
    class ServiceClient < RestClientBase
      
      def initialize(host, port)
        super(host, port)
      end
      
      def post_validation_result(deployment_id, rule_id, node_id, message)
        private_ip = Socket.ip_address_list.detect{|i| i.ipv4? && !i.ipv4_loopback? && !i.ipv4_multicast? && i.ipv4_private?}.ip_address
        params = {:deployment_id => deployment_id, :rule_id => rule_id, :node_id => node_id, :message => message, :node_private_ip => private_ip}
        #$logger.debug "Posting the following validation error info: #{params.inspect}"
        response = post(SERVICE_VALIDATION_URI, params)
        if (response.code == 201)
          json = JSON.parse(response.body)
          return json["id"]
        else
          raise WeaverServiceError.new("Failed to post validation, response code is #{response.code}", response.code)
        end        
      end
      
      def generate_check_process_rule(weaver_project_dir, node_id, pid, user, full_command_line)
        state_info = {}
        state_info[:pid] = pid
        state_info[:checker_arguments] = "#{user} '#{full_command_line}'"
          
        params = {:weaver_project_dir => weaver_project_dir, :node_id => node_id, :state_type => :process, :state_info => state_info}
        response = post(SERVICE_RULE_URI, params)
        if (response.code == 201)
          json = JSON.parse(response.body)
          return json["id"]
        else
          raise WeaverServiceError.new("Failed to post rule, response code is #{response.code}", response.code)
        end        
      end
      
      def generate_check_port_rule(weaver_project_dir, node_id, ports)
        state_info = {}
        state_info[:checker_arguments] = ports.join(' ')

        params = {:weaver_project_dir => weaver_project_dir, :node_id => node_id, :state_type => :port, :state_info => state_info}
        response = post(SERVICE_RULE_URI, params)
        if (response.code == 201)
          json = JSON.parse(response.body)
          return json["id"]
        else
          raise WeaverServiceError.new("Failed to post rule, response code is #{response.code}", response.code)
        end
      end
    end
  end
end